from django.conf import settings

RECOMMENDS_STORAGE_REDIS_DATABASE_DEFAULT = {
    'HOST': 'localhost',
    'PORT': 6379,
    'NAME': 0
}

RECOMMENDS_STORAGE_REDIS_DATABASE = getattr(settings, 'RECOMMENDS_STORAGE_REDIS_DATABASE', RECOMMENDS_STORAGE_REDIS_DATABASE_DEFAULT)
